/************************************************************************
 * SGA - A C++ library to help develop Simple Genetic Algorithms        *
 * Copyright (C) 2005 Dorival M. Pedroso                                *
 *                                                                      *
 * This program is free software: you can redistribute it and/or modify *
 * it under the terms of the GNU General Public License as published by *
 * the Free Software Foundation, either version 3 of the License, or    *
 * any later version.                                                   *
 *                                                                      *
 * This program is distributed in the hope that it will be useful,      *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of       *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the         *
 * GNU General Public License for more details.                         *
 *                                                                      *
 * You should have received a copy of the GNU General Public License    *
 * along with this program. If not, see <http://www.gnu.org/licenses/>  *
 ************************************************************************/

// STL
#include <stdio.h>
#include <string.h>
#include <emmintrin.h>
#include <inttypes.h>

// 128-bit data structure (for SSE2 only)
union W128_T {
	__m128i  si;
	__m128d  sd;
	uint64_t u[2];
	uint32_t u32[4];
	double   d[2];
};

// Array structures for SSE2
#define BLOCK_SIZE 100000
static __m128i array1[BLOCK_SIZE / 4];
static __m128i array2[10000 / 4];

// SGA
#include "random.h"

extern "C"
{
	int    get_min_array_size32    (void);
	char * get_idstring            (void);
	void   fill_array32            (uint32_t * array, int size);
	void   init_by_array           (uint32_t * init_key, int key_length);
	void   fill_array_close1_open2 (double array[], int size);
}

int main(int argc, char* argv[])
{
	//////////////////////////////////////////////////////////////////////////////// SFMT /////
	
	{
		int i;
		uint32_t *array32 = (uint32_t *)array1;
		uint32_t *array32_2 = (uint32_t *)array2;
		uint32_t ini[4] = {0x1234, 0x5678, 0x9abc, 0xdef0};
		uint32_t r32;

		if (get_min_array_size32() > 10000) {
		printf("array size too small!\n");
		exit(1);
		}
		printf("%s\n32 bit generated randoms\n", get_idstring());
		printf("init_gen_rand__________\n");
		/* 32 bit generation */
		Rnd::Init(1234);
		fill_array32(array32, 10000);
		fill_array32(array32_2, 10000);
		Rnd::Init(1234);
		for (i = 0; i < 10000; i++) {
		if (i < 1000) {
			printf("%10u ", array32[i]);
			if (i % 5 == 4) {
			printf("\n");
			}
		}
		r32 = gen_rand32();
		if (r32 != array32[i]) {
			printf("\nmismatch at %d array32:%x gen:%x\n", 
			   i, array32[i], r32);
			exit(1);
		}
		}
		for (i = 0; i < 700; i++) {
		r32 = gen_rand32();
		if (r32 != array32_2[i]) {
			printf("\nmismatch at %d array32_2:%x gen:%x\n", 
			   i, array32_2[i], r32);
			exit(1);
		}
		}
		printf("\n");
		init_by_array(ini, 4);
		printf("init_by_array__________\n");
		fill_array32(array32, 10000);
		fill_array32(array32_2, 10000);
		init_by_array(ini, 4);
		for (i = 0; i < 10000; i++) {
		if (i < 1000) {
			printf("%10u ", array32[i]);
			if (i % 5 == 4) {
			printf("\n");
			}
		}
		r32 = gen_rand32();
		if (r32 != array32[i]) {
			printf("\nmismatch at %d array32:%x gen:%x\n", 
			   i, array32[i], r32);
			exit(1);
		}
		}
		for (i = 0; i < 700; i++) {
		r32 = gen_rand32();
		if (r32 != array32_2[i]) {
			printf("\nmismatch at %d array32_2:%x gen:%x\n", 
			   i, array32_2[i], r32);
			exit(1);
		}
		}
	}

	/////////////////////////////////////////////////////////////////////////////// dSFMT /////
	
	{
		const int NUM_RANDS = 50000;
		W128_T dummy[NUM_RANDS / 2 + 1];

		int i;
		union W64_T { uint64_t u; double d; };
		union W64_T *array = (union W64_T *)dummy;
		union W64_T r;

		printf("generated randoms [1, 2)\n");
		Rnd::Init(1234);
		fill_array_close1_open2(&array[0].d, 5000);
		Rnd::Init(1234);
		for (i = 0; i < 5000; i++)
		{
			r.d = genrand_close1_open2();
			if (r.d != array[i].d)
			{
				printf("\n[1, 2) mismatch\n");
				exit(1);
			}
			if (i < 1000)
			{
				printf("%1.20lf ", array[i].d);
				if (i % 3 == 2) printf("\n");
			}
		}
		printf("\n");
	}
}
